
(* SPAN - A Security Protocol Animator
   Copyright (C) 2006-2008 Yann Glouche, Thomas Genet and Erwan Houssay

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.5. Last modification 23/09/08
*)
open Utils_simulator

(**
automate.ml
@author Yann GLOUCHE
@version 1.0
@since 12/12/2005
*)

(** Automate and automatic historic *)


(** in a pair (state, message), findIknows return the message 
 @param s is a pair of string  (state, message)
 @return string
 *)
 let findIknows (s : string * string) : string = match s with (_,i) -> i;;
(** in a pair (state,message), findIknows return the state
 @param s is a pair of string  (state, message)
 @return string
 *)			
 let findState (s : string * string) : string = match s with (e,_) -> e;;
(**
the automate is a very important structure. automate describes the current state of all the agents of the protocol.
automate is a list of uplet where each uplet is an instanciation of a role.

The form of the uplet is : (name_of_the_agent, number_of_the_instanciation, ([(current_state, bool ref);...list] ref),[ ((state,received_message),(([left_conditions;...list],[left_affectations;...list]),([right_conditions;...lists],[right_affectations;...list])),(state,sent_message))list_of_transitions])

if !bool=true then the received_message is not received else the received_message is received
*)
let automate = (ref [] : 	(string * int * (string * bool Pervasives.ref) list Pervasives.ref *
        			((string * string) *
        			 	((string list * string list) * (string list * string list)) *
        			 (string * string))
       				 list)
       			list Pervasives.ref);;
(**
contains the historic of the automate. The transition are not save in this structure (it is not an interesting thing)
*)
let automate_story = (ref [] : (string * int * (string * bool) list) list list Pervasives.ref);;
(**
save the current state of the automate in automate_story
@return unit
*)
let add_in_automate_story (() : unit) : unit =
 	let rec last_state auto res =
		match auto with
			[] -> (List.rev res)
			|((name,id,statesL,_)::ls) -> last_state ls ((name,id,(List.map (fun (state,exec) -> (state,(!exec))) (!statesL)))::res)
	in
	automate_story:=((last_state (!automate) [])::(!automate_story))
;;
(** 
load the previous state of the automate
@return unit 
*)
let rec load_last_state (x : (string * int * (string * bool) list) list) 
			(automate : 	(string * int * (string * bool Pervasives.ref) list Pervasives.ref * 'a ) list) : unit = 
	let load_statesL states_list l_ref = l_ref:=(List.map (fun (state,exec)-> (state, ref exec)) states_list) in
	let rec find_state name id auto =
		match auto with [] -> failwith "Bad automate"
				|((name2,id2,statesL,_)::ls) -> if(name=name2 && id=id2) then statesL
								else find_state name id ls;
	in
	let rec aux x auto =
	match x with
		[] -> () 
		|((name,id,states_list)::xs) -> (
			match auto with [] -> ()
					|((name2,id2,statesL,_)::ls) -> if(name=name2 && id=id2) then (load_statesL states_list statesL; aux xs ls)
									else (
									let statesLL = (find_state name id ls) in 
									load_statesL states_list statesLL;
									aux xs auto);
			);
	in
	aux x automate
;;

(**
the automate comme back to the previous state
@return unit
*)
let previous_state (() : unit) : unit =
	match (!automate_story) with
		(x::ls) -> 
			load_last_state x (!automate);
			automate_story:=ls
		|_ -> ()
;;

let max_length_message = ref (-1);;

let get_max_length_message () = 
	if((!max_length_message) < 0) 
	then(
		let all_actions = List.flatten (List.map (fun (_,_,_,actionsL) -> actionsL) (!automate)) in
		let all_messages = List.flatten (List.map (fun (initial,_,final) -> [(findIknows initial);(findIknows final)]) all_actions) in
		let max_length = max_length_string all_messages in
		max_length_message := max_length;
		max_length	
	)
	else (!max_length_message)
;;
