
(* SPAN - A Security Protocol Animator
   Copyright (C) 2006-2008 Yann Glouche, Thomas Genet and Erwan Houssay

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.5. Last modification 23/09/08
*)
(* ..................................................................
    Module.......: ErrorHandler_unify
    Initial Date : 14 Octobre 2003
    Last Update..: 17 March 2004
    Developer....: Judson Santiago
    Maintenance..: Judson Santiago
    Compiler.....: OCAML 3.07 
    Comments.....: Handle syntax, semantic and compilation errors
 .................................................................... *)


(* Keeping track of the current line number of file being parsed  *)
let lineCounter = ref 1;;

(* A unique error number to each error *)
let errorNumber = ref 0;;

(* This exception is raised everytime the IF file has some unsoported
   feature. It is beeing used only to the Exponentiation operator for
   the moment *)
exception Unsupported;;

(* Handling system errors *)
let systemError errorMessage=
  print_endline (errorMessage);
  exit(-1)
;;

(* Handling lexer errors. It generates appropriate error messages
   containing the line number where the error has occurred *)
let lexerError lexerBuffer errorMessage=
  print_string "Line "; print_int !lineCounter;
  print_string ": Lex.Err: ";
  print_string errorMessage; print_string " ";
  print_endline (lexerBuffer);
  exit (-1)
;;


(* Handling parser errors. It generates error messages according to
the reported error numbers *)
let parserError() =
  print_string "Line "; print_int !lineCounter;
  print_string ":Parser.Err("; print_int !errorNumber; print_string ")";

  (match !errorNumber
  with
    | 1 -> print_endline ": missing State Identifier"
    | 2 -> print_endline ": missing DOT Symbol"
    | 3 -> print_endline ": missing Inits Section keyword"
    | 4 -> print_endline ": missing Init State keyword"
    | 5 -> print_endline ": missing Identifier"
    | 6 -> print_endline ": missing Symbol DEF (:=)"
    | 7 -> print_endline ": missing Step keyword"
    | 8 -> print_endline ": missing left parenteses"
    | 9 -> print_endline ": missing right parenteses"
    | 10 -> print_endline ": missing Variable"
    | 11 -> print_endline ": missing Symbol REACTION (=>)"
    | 12 -> print_endline ": missing Section Rules keyword"
    | 13 -> print_endline ": missing Term"
    | 14 -> print_endline ": missing Type"
    | 15 -> print_endline ": missing Comma"
    | _ -> print_endline ": unknown parsing error");
  flush stdout;
  exit (-1)
;;
