
(* SPAN - A Security Protocol Animator
   Copyright (C) 2006-2008 Yann Glouche, Thomas Genet and Erwan Houssay

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.5. Last modification 23/09/08
*)
(**
message_stack.ml
@author Yann GLOUCHE
@version 1.0
@since 12/12/2005
*)

 open Utils_simulator;;
(** contains all the messages to send and to receive in the waiting line. The value of the hash table is a message, and the key is a 3-uplet (name_of_an_agent, state_of_the_agent, string_value) where string_value is a string equal to "_rcv_stack" for indicate the messages to wait, or string_value is equal to "_message_stack" for indicate the message to send *)
 let message_stack = (Hashtbl.create 600 : (string * string * string, string) Hashtbl.t);;
 (** contains all the messages sent and received in the waiting line since the begin of the protocol execution. each element is a step of the protocol : all the messages sent and received in the waiting line in this step of the protocol. it is use for create an historic *)
 let remove_stack_story = (ref [] : ((string * string * string) * string) list list Pervasives.ref);;
  (** contains all the messages to send and to receive in the waiting line since the begin of the protocol execution. each element is a step of the protocol :  all the messages to send and to received in the waiting line in this step of the protocol. it is use for create an historic *)
 let add_stack_story = (ref [] : ((string * string * string) * string) list list Pervasives.ref);;
 (** contains all the messages sent and received in the waiting line in the current step of the protocol *)
 let remove_of_stack = (ref [] : ((string * string * string) * string) list Pervasives.ref);;
  (** contains all the messages to send and to receive in the waiting line in the current step of the protocol *)
 let add_on_stack = (ref[] : ((string * string * string) * string) list Pervasives.ref);;
(**
add the current value add_on_stack in add_stack_story : add the current value to the historic,
and initialize the value of add_on_stack
*)
 let next_add_stack (() : unit) : unit = add_stack_story := ((List.rev (!add_on_stack))::(!add_stack_story));
 					 add_on_stack:=[];;
(**
 the variables add_stack_story and add_on_stack comme back to her previous value in the previous step of the protocol.
*)
 let previous_add_stack (() : unit) : unit = match (!add_stack_story) with [] -> ()
 							|(x::ls) -> 	(List.iter
									(fun (name_in_session,message) -> (
										let message_list = Hashtbl.find_all message_stack name_in_session
										in
										let rec aux l buff =
											match l with [] -> (List.iter (fun t -> (Hashtbl.add message_stack name_in_session t)) buff)
													|(x::ls) -> (	Hashtbl.remove message_stack name_in_session;
														if (x=message)
														then(aux ls buff)
														else(aux ls (x::buff)));
										in
										aux message_list [];							
									)) (!add_on_stack);
									add_on_stack:=x;
									add_stack_story:=ls);;
(**
add the current value remove_of_stack in remove_stack_story : add the current value to the historic,
and initialize the value of remove_of_stack
*)						
 let next_remove_stack (() : unit) : unit = 	remove_stack_story := ((List.rev (!remove_of_stack))::(!remove_stack_story));
 				 remove_of_stack:=[];;
(**
 the variables remove_stack_story and remove_of_stack comme back to her previous value in the previous step of the protocol.
*)				 
 let previous_remove_stack (() : unit) :unit = match (!remove_stack_story) with [] -> ()
 							|(x::ls) -> 	(List.iter
									(fun (name_in_session,message) -> Hashtbl.add message_stack name_in_session message)
									(!remove_of_stack);
									remove_of_stack:=x;
									remove_stack_story:=ls);;
 (**
 This function call next_add_stack () and next_remove_stack (), then this function :
 - add the current value add_on_stack in add_stack_story : add the current value to the historic,
and initialize the value of add_on_stack
 - add the current value remove_of_stack in remove_stack_story : add the current value to the historic,
and initialize the value of remove_of_stack
 *)
 let next_message_stack (() : unit) : unit = 
 	next_add_stack (); 
	next_remove_stack ();;
(**
 the variables add_stack_story and add_on_stack comme back to her previous value in the previous step of the protocol, and
 the variables remove_stack_story and remove_of_stack comme back to her previous value in the previous step of the protocol.
*)
 let previous_message_stack (() : unit) :unit =
 	previous_add_stack ();
	previous_remove_stack ();;
(** test if the message "message" in the stack of messages to send by the agent "name_in_session" in the state "state" is in the hash table message_stack.
@return unit
*)					
 let in_message_stack (name_in_session : string) (state : string) (message : string) : bool =
	Hashtbl.mem message_stack (name_in_session,state,"_message_stack");;
	
(** add the message "message" in the stack of messages to send by the agent "name_in_session" in the state "state". 
@return unit
*)					
 let add_in_message_stack (name_in_session : string) (state : string) (message : string) : unit =
	let message_stack_name = (name_in_session,state,"_message_stack") in
	
	if(List.mem (message_stack_name,message) (!remove_of_stack))
	then remove_of_stack:=remove (message_stack_name,message) (!remove_of_stack)
	else add_on_stack:= (message_stack_name,message)::(!add_on_stack);
	
	Hashtbl.add message_stack message_stack_name message;;

(** return the stack of messages to send by the agent "name_in_session" in the state "state". 
@return a list of messages
*)	
 let get_message_stack (name_in_session : string) (state : string) : string list = 
 	Hashtbl.find_all message_stack (name_in_session,state,"_message_stack");;
 
(** test if the stack of messages to send is empty for the agent "name_in_session" in the state "state". 
@return true is the stack of messages to send is emplty, else false
*)
 let message_stack_is_empty (name_in_session : string) (state : string) : bool = ((get_message_stack name_in_session state)=[]);;
 (** delete the message "message" in the stack of messages to send by the agent name_in_session in the state "state". 
@return true is the message is deleted, else false is the message is not deleted or if the message is not in the stack
*)
 let remove_message_stack (name_in_session : string) (state : string) (message : string) : bool = 
 	let res = ref false in
  	let message_list = get_message_stack name_in_session state in
	let key = (name_in_session,state,"_message_stack") in
	let rec aux l buff =
		match l with [] -> (List.iter (fun t -> (Hashtbl.add message_stack key t)) buff)
			|(x::ls) -> (	Hashtbl.remove message_stack key;
					if (x=message)
					then(res:=true ; aux ls buff)
					else(aux ls (x::buff)));
	in
	aux message_list [];
	if(!res) then 
		(	if(List.mem (key,message) (!add_on_stack))
			then add_on_stack:=remove (key,message) (!add_on_stack)
			else remove_of_stack:= (key,message)::(!remove_of_stack)
		);
	(!res)
	;;
(** delete all the messages "message" in the stack of the messages to send by the agent "name_in_session" in the state "state". 
@return unit
*)
 let remove_state_message_stack (name_in_session : string) (state : string) : unit =
	let remove_message n message = 
		let res = ref false in
		let message_list = Hashtbl.find_all message_stack n in
		let rec aux l buff =
			match l with [] -> (List.iter (fun t -> (Hashtbl.add message_stack n t)) buff)
				|(x::ls) -> (	Hashtbl.remove message_stack n;
						if (x=message)
						then(res:=true ; aux ls buff)
						else(aux ls (x::buff)));
		in
		aux message_list [];
		if(!res) then		
		(	if(List.mem (n,message) (!add_on_stack))
			then add_on_stack:=remove (n,message) (!add_on_stack)
			else remove_of_stack:= (n,message)::(!remove_of_stack)
		);
	in
	(Hashtbl.iter
	(fun n message -> if((name_in_session,state,"_message_stack")=n) then  (remove_message n message))
	message_stack)
;;
(** delete all the messages "message" in the stack of the messages to send by the "agent name_in_session" in all the states of this agent. 
@return unit
*)
 let remove_all_message_stack (name_in_session : string) : unit =
	let remove_message n message = 
		let res = ref false in
		let message_list = Hashtbl.find_all message_stack n in
		let rec aux l buff =
			match l with [] -> (List.iter (fun t -> (Hashtbl.add message_stack n t)) buff)
				|(x::ls) -> (	Hashtbl.remove message_stack n;
						if (x=message)
						then(res:= true ; aux ls buff)
						else(aux ls (x::buff)));
		in
		aux message_list [];
		if(!res) then		
		(	if(List.mem (n,message) (!add_on_stack))
			then add_on_stack:=remove (n,message) (!add_on_stack)
			else remove_of_stack:= (n,message)::(!remove_of_stack)
		);
	in
	(Hashtbl.iter
	(fun (name,state,type_of_mess) message -> if(name=name_in_session && type_of_mess="_message_stack") then  (remove_message (name,state,type_of_mess)  message))
	message_stack)
;;
(** add the message "message" in the stack of messages to receive by the agent "name_in_session" in the state "state". 
@return unit
*)		
 let add_in_rcv_stack (name_in_session : string) (state : string) (message : string) : unit =
	let message_stack_name = (name_in_session,state,"_rcv_stack") in
	
	if(List.mem (message_stack_name,message) (!remove_of_stack))
	then remove_of_stack:=remove (message_stack_name,message) (!remove_of_stack)
	else add_on_stack:= (message_stack_name,message)::(!add_on_stack);
	
	Hashtbl.add message_stack message_stack_name message;;

(** return the stack of messages to receive by the agent "name_in_session" in the state "state". 
@return a list of messages
*)
 let get_rcv_stack (name_in_session : string) (state : string) : string list = 
 	Hashtbl.find_all message_stack (name_in_session,state,"_rcv_stack");;
 
(** test if the stack of messages to receive is empty for the agent "name_in_session" in the state "state". 
@return true is the stack of messages to receive is emplty, else false
*)
 let rcv_stack_is_empty (name_in_session : string) (state : string) : bool = ((get_rcv_stack name_in_session state)=[]);;
  (** delete the message "message" in the stack of messages to receive by the agent "name_in_session" in the state "state". 
@return true is the message is deleted, else false is the message is not deleted or if the message is not in the stack
*)	
 let remove_rcv_stack (name_in_session : string) (state : string) (message : string) : bool = 
 	let res = ref false in
  	let message_list = get_rcv_stack name_in_session state in
	let key = (name_in_session,state,"_rcv_stack") in
	let rec aux l buff =
		match l with [] -> 	(Hashtbl.remove message_stack key;
					List.iter (fun t -> (Hashtbl.add message_stack key t)) buff)
			|(x::ls) -> (	
					if (x=message)
					then( res:=true ; aux ls buff)
					else(aux ls (x::buff)));
	in
	aux message_list [];
	if(!res) then		
		(	if(List.mem (key,message) (!add_on_stack))
			then add_on_stack:=remove (key,message) (!add_on_stack)
			else remove_of_stack:= (key,message)::(!remove_of_stack)
		);
	(!res);
	;;
(** delete all the messages "message" in the stack of the messages to receive by the agent "name_in_session" in the state "state". 
@return unit
*)
let remove_state_rcv_stack (name_in_session : string) (state : string) : unit =
	let remove_message n message = 
		let res = ref false in
		let message_list = Hashtbl.find_all message_stack n in
		let rec aux l buff =
			match l with [] -> (List.iter (fun t -> (Hashtbl.add message_stack n t)) buff)
				|(x::ls) -> (	Hashtbl.remove message_stack n;
						if (x=message)
						then(res:=true ; aux ls buff)
						else(aux ls (x::buff)));
		in
		aux message_list [];
		if(!res) then		
		(	if(List.mem (n,message) (!add_on_stack))
			then add_on_stack:=remove (n,message) (!add_on_stack)
			else remove_of_stack:= (n,message)::(!remove_of_stack)
		);
	in
	(Hashtbl.iter
	(fun n message -> if((name_in_session,state,"_rcv_stack")=n) then  (remove_message n message))
	message_stack)
;;
(** delete all the messages "message" in the stack of the messages to receive by the agent "name_in_session" in all the states of this agent. 
@return unit
*)
 let remove_all_rcv_stack (name_in_session : string) : unit =
	let remove_message n message = 
		let res = ref false in
		let message_list = Hashtbl.find_all message_stack n in
		let rec aux l buff =
			match l with [] -> 	(List.iter (fun t -> (Hashtbl.add message_stack n t)) buff)
				|(x::ls) -> (	Hashtbl.remove message_stack n;
						if (x=message)
						then(res:= true ; aux ls buff)
						else(aux ls (x::buff)));
		in
		aux message_list [];
		if(!res) then		
		(	if(List.mem (n,message) (!add_on_stack))
			then add_on_stack:=remove (n,message) (!add_on_stack)
			else remove_of_stack:= (n,message)::(!remove_of_stack)
		);
	in
	(Hashtbl.iter
	(fun (name,state,type_of_mess) message -> if(name=name_in_session && type_of_mess="_rcv_stack") then  (remove_message (name,state,type_of_mess) message))
	message_stack)
;;
